@@ -73,7 +73,7 @@ import sabnzbd.dirscanner as dirscanner
 import sabnzbd.urlgrabber as urlgrabber
 import sabnzbd.scheduler as scheduler
 import sabnzbd.rss as rss
-import sabnzbd.email as email
+import sabnzbd.emailer as emailer
 import sabnzbd.articlecache as articlecache
 import sabnzbd.newsunpack
 import sabnzbd.codecs as codecs
@@ -111,6 +111,7 @@ LOGFILE = None
 WEBLOGFILE = None
 LOGHANDLER = None
 GUIHANDLER = None
+LOG_ALL = False
 AMBI_LOCALHOST = False
 
 WEB_DIR = None
@@ -213,6 +214,8 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False):
     if not os.path.exists(os.path.join(cfg.CACHE_DIR.get_path(), QUEUE_FILE_NAME)):
         OLD_QUEUE = bool(glob.glob(os.path.join(cfg.CACHE_DIR.get_path(), QUEUE_FILE_TMPL % '?')))
 
+    sabnzbd.change_queue_complete_action(cfg.queue_complete.get(), new=False)
+
     ###
     ### Initialize threads
     ###
@@ -356,8 +359,6 @@ def add_msgid(msgid, pp=None, script=None, cat=None, priority=None, nzbname=None
     if pp and pp=="-1": pp = None
     if script and script.lower()=='default': script = None
     if cat and cat.lower()=='default': cat = None
-    if priority == None: priority = NORMAL_PRIORITY
-
 
     if cfg.USERNAME_NEWZBIN.get() and cfg.PASSWORD_NEWZBIN.get():
         logging.info('Fetching msgid %s from www.newzbin.com', msgid)
@@ -376,8 +377,6 @@ def add_url(url, pp=None, script=None, cat=None, priority=None, nzbname=None):
     if pp and pp=="-1": pp = None
     if script and script.lower()=='default': script = None
     if cat and cat.lower()=='default': cat = None
-    if 'nzbindex.nl/' in url or 'nzbindex.com/' in url:
-        nzbname = ''
     logging.info('Fetching %s', url)
     msg = T('fetchNZB@1') % url
     future_nzo = nzbqueue.generate_future(msg, pp, script, cat, url=url, priority=priority, nzbname=nzbname)
@@ -439,7 +438,7 @@ def backup_nzb(filename, data):
             _f.close()
         except:
             logging.error("Saving %s to %s failed", backup_name, cfg.NZB_BACKUP_DIR.get_path())
-            logging.debug("Traceback: ", exc_info = True)
+            logging.info("Traceback: ", exc_info = True)
 
         os.chdir(here)
 
@@ -473,7 +472,7 @@ def add_nzbfile(nzbfile, pp=None, script=None, cat=None, priority=NORMAL_PRIORIT
         os.close(f)
     except:
         logging.error(Ta('error-tempFile@1'), filename)
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
 
     if ext.lower() in ('.zip', '.rar'):
         dirscanner.ProcessArchiveFile(filename, path, pp, script, cat, priority=priority)
@@ -557,10 +556,11 @@ def restart_program():
     cherrypy.engine.restart()
 
 
-def change_queue_complete_action(action):
+def change_queue_complete_action(action, new=True):
     """
     Action or script to be performed once the queue has been completed
     Scripts are prefixed with 'script_'
+    When "new" is False, check wether non-script actions are acceptable
     """
     global QUEUECOMPLETE, QUEUECOMPLETEACTION, QUEUECOMPLETEARG
 
@@ -570,14 +570,19 @@ def change_queue_complete_action(action):
         #all scripts are labeled script_xxx
         _action = run_script
         _argument = action.replace('script_', '')
-    elif action == 'shutdown_pc':
-        _action = system_shutdown
-    elif action == 'hibernate_pc':
-        _action = system_hibernate
-    elif action == 'standby_pc':
-        _action = system_standby
-    elif action == 'shutdown_program':
-        _action = shutdown_program
+    elif new or cfg.queue_complete_pers.get():
+        if action == 'shutdown_pc':
+            _action = system_shutdown
+        elif action == 'hibernate_pc':
+            _action = system_hibernate
+        elif action == 'standby_pc':
+            _action = system_standby
+        elif action == 'shutdown_program':
+            _action = shutdown_program
+        else:
+            action = None
+    else:
+        action = None
 
     #keep the name of the action for matching the current select in queue.tmpl
     QUEUECOMPLETE = action
@@ -617,7 +622,7 @@ def CheckFreeSpace():
             logging.warning(Ta('warn-noSpace'))
             # Pause downloader, but don't save, since the disk is almost full!
             downloader.pause_downloader(save=False)
-            email.diskfull()
+            emailer.diskfull()
 
 
 ################################################################################
@@ -634,13 +639,14 @@ def get_new_id(prefix):
         return tail
     except:
         logging.error(Ta('error-failMkstemp'))
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
 
 
 @synchronized(IO_LOCK)
-def save_data(data, _id, do_pickle = True, doze=0):
+def save_data(data, _id, do_pickle = True, doze=0, silent=False):
     path = os.path.join(cfg.CACHE_DIR.get_path(), _id)
-    logging.info("Saving data for %s in %s", _id, path)
+    if not silent:
+        logging.debug("Saving data for %s in %s", _id, path)
 
     try:
         _f = open(path, 'wb')
@@ -655,13 +661,14 @@ def save_data(data, _id, do_pickle = True, doze=0):
         _f.close()
     except:
         logging.error(Ta('error-saveX@1'), path)
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
 
 
 @synchronized(IO_LOCK)
-def load_data(_id, remove = True, do_pickle = True):
+def load_data(_id, remove = True, do_pickle = True, silent=False):
     path = os.path.join(cfg.CACHE_DIR.get_path(), _id)
-    logging.info("Loading data for %s from %s", _id, path)
+    if not silent:
+        logging.debug("Loading data for %s from %s", _id, path)
 
     if not os.path.exists(path):
         logging.info("%s missing", path)
@@ -681,7 +688,7 @@ def load_data(_id, remove = True, do_pickle = True):
             remove_data(_id)
     except:
         logging.error(Ta('error-loading@1'), path)
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
         return None
 
     return data
@@ -745,6 +752,9 @@ def check_all_tasks():
     # Kick the downloader, in case it missed the semaphore
     sabnzbd.downloader.wakeup()
 
+    # Make sure the right servers are active
+    sabnzbd.downloader.check()
+
     # Restartable threads
     if not sabnzbd.dirscanner.alive():
         logging.info('Restarting crashed dirscanner')
