@@ -55,7 +55,8 @@ class ArticleCache:
         if nzf.deleted or nzo.deleted:
             # Do not discard this article because the
             # file might still be processed at this moment!!
-            logging.info("%s would be discarded", article)
+            if sabnzbd.LOG_ALL:
+                logging.debug("%s would be discarded", article)
             # return
 
         saved_articles = article.nzf.nzo.saved_articles
@@ -97,11 +98,11 @@ class ArticleCache:
             data = self.__article_table.pop(article)
             self.__article_list.remove(article)
             self.__cache_size -= len(data)
-            logging.info("Loaded %s from cache", article)
-            logging.debug("cache_size -> %s", self.__cache_size)
+            if sabnzbd.LOG_ALL:
+                logging.debug("Loaded %s from cache", article)
         elif article.art_id:
             data = sabnzbd.load_data(article.art_id, remove = True,
-                                     do_pickle = False)
+                                     do_pickle = False, silent=True)
 
         nzo = article.nzf.nzo
         if article in nzo.saved_articles:
@@ -119,7 +120,8 @@ class ArticleCache:
 
     @synchronized(ARTICLE_LOCK)
     def purge_articles(self, articles):
-        logging.debug("Purgable articles -> %s", articles)
+        if sabnzbd.LOG_ALL:
+            logging.debug("Purgable articles -> %s", articles)
         for article in articles:
             if article in self.__article_list:
                 self.__article_list.remove(article)
@@ -135,14 +137,15 @@ class ArticleCache:
         if nzf.deleted or nzo.deleted:
             # Do not discard this article because the
             # file might still be processed at this moment!!
-            logging.info("%s would be discarded", article)
+            if sabnzbd.LOG_ALL:
+                logging.debug("%s would be discarded", article)
             # return
 
         art_id = article.get_art_id()
         if art_id:
-            logging.info("Flushing %s to disk", article)
-            logging.debug("cache_size -> %s", self.__cache_size)
-            sabnzbd.save_data(data, art_id, do_pickle = False, doze=self.__doze)
+            if sabnzbd.LOG_ALL:
+                logging.debug("Flushing %s to disk", article)
+            sabnzbd.save_data(data, art_id, do_pickle = False, doze=self.__doze, silent=True)
         else:
             logging.warning("Flushing %s failed -> no art_id", article)
 
@@ -154,8 +157,8 @@ class ArticleCache:
 
         self.__article_table[article] = data
         self.__cache_size += len(data)
-        logging.info("Added %s to cache", article)
-        logging.debug("cache_size -> %s", self.__cache_size)
+        if sabnzbd.LOG_ALL:
+            logging.debug("Added %s to cache", article)
 
 
 ### Global access point for article cache
