@@ -124,6 +124,9 @@ class Assembler(Thread):
                             sabnzbd.downloader.pause_downloader()
                         else:
                             logging.error(Ta('error-diskError@1'), filepath)
+                    except:
+                        logging.error('Fatal error in Assembler', exc_info = True)
+                        break
 
                     setname = nzf.get_setname()
                     if nzf.is_par2() and (nzo.get_md5pack(setname) is None):
@@ -228,8 +231,8 @@ def GetMD5Hashes(fname):
         logging.info('Cannot use corrupt par2 file for QuickCheck, "%s"', fname)
         table = {}
     except:
-        logging.debug('QuickCheck parser crashed in file %s', fname)
-        logging.debug('Traceback: ', exc_info = True)
+        logging.info('QuickCheck parser crashed in file %s', fname)
+        logging.info('Traceback: ', exc_info = True)
         table = {}
 
     f.close()
