@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2010 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -78,6 +78,9 @@ class Option:
         else:
             return self.__default_val
 
+    def get_string(self):
+        return str(self.get())
+
     def get_dict(self, safe=False):
         """ Return value a dictionary """
         return { self.__keyword : self.get() }
@@ -191,9 +194,9 @@ class OptionDir(Option):
         if value != None and value != self.get():
             value = value.strip()
             if self.__validation:
-                error, val = self.__validation(self.__root, value)
+                error, value = self.__validation(self.__root, value, self._Option__default_val)
             if not error:
-                if self.__create:
+                if value and self.__create:
                     res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask)
                     if not res:
                         error = "Cannot create %s folder %s" % (self.ident()[1], path)
@@ -203,18 +206,23 @@ class OptionDir(Option):
 
 class OptionList(Option):
     """ List option class """
-    def __init__(self, section, keyword, default_val=None, add=True):
+    def __init__(self, section, keyword, default_val=None, validation=None, add=True):
+        self.__validation = validation
         if default_val is None:
             default_val = []
         Option.__init__(self, section, keyword, default_val, add=add)
 
     def set(self, value):
-        """ Set value, convert single item to list of one """
-        if value != None:
-            if type(value) != type([]):
-                value = [ value ]
-            return self._Option__set(value)
-        return None
+        """ Set the list given a comma-separated string or a list"""
+        error = None
+        if value is not None:
+            if not isinstance(value, list):
+                value = listquote.simplelist(value)
+            if self.__validation:
+                error, value = self.__validation(value)
+            if not error:
+                self._Option__set(value)
+        return error
 
     def get_string(self):
         """ Return the list as a comma-separated string """
@@ -228,12 +236,6 @@ class OptionList(Option):
             if n < r-1: txt += ', '
         return txt
 
-    def set_string(self, txt):
-        """ Set the list given a comma-separated string """
-        if type(txt) == type(''):
-            self._Option__set(listquote.simplelist(txt))
-        else:
-            self._Option__set(txt)
 
 
 class OptionStr(Option):
@@ -268,6 +270,7 @@ class OptionPassword(Option):
     """ Password class """
     def __init__(self, section, keyword, default_val='', add=True):
         Option.__init__(self, section, keyword, default_val, add=add)
+        self.get_string = self.get_stars
 
     def get(self):
         """ Return decoded password """
@@ -401,10 +404,7 @@ class ConfigCat:
                 value = values[kw]
             except KeyError:
                 continue
-            if kw == 'newzbin':
-                exec 'self.%s.set_string(value)' % kw
-            else:
-                exec 'self.%s.set(value)' % kw
+            exec 'self.%s.set(value)' % kw
         return True
 
     def get_dict(self, safe=False):
@@ -585,7 +585,7 @@ def get_config(section, keyword):
     try:
         return database[section][keyword]
     except KeyError:
-        logging.info('Missing configuration item %s,%s', section, keyword)
+        logging.debug('Missing configuration item %s,%s', section, keyword)
         return None
 
 
@@ -626,9 +626,13 @@ def read_config(path):
     if not os.path.exists(path):
         # No file found, create default INI file
         try:
+            if not sabnzbd.WIN32:
+                prev= os.umask(077)
             fp = open(path, "w")
             fp.write("__version__=%s\n[misc]\n[logging]\n" % __CONFIG_VERSION)
             fp.close()
+            if not sabnzbd.WIN32:
+                os.umask(prev)
         except IOError:
             return False, 'Cannot create INI file %s' % path
 
@@ -682,7 +686,10 @@ def save_config(force=False):
             except:
                 CFG[section] = {}
             for subsec in database[section]:
-                subsec_mod = subsec.replace('[', '{').replace(']','}')
+                if section == 'servers':
+                    subsec_mod = subsec.replace('[', '{').replace(']','}')
+                else:
+                    subsec_mod = subsec
                 try:
                     CFG[section][subsec_mod]
                 except:
@@ -849,24 +856,24 @@ def validate_octal(value):
         return Ta('error-notOctal@1') % value, None
 
 
-def validate_no_unc(root, value):
+def validate_no_unc(root, value, default):
     """ Check if path isn't a UNC path """
     # Only need to check the 'value' part
     if value and not value.startswith(r'\\'):
-        return None, value
+        return validate_notempty(root, value, default)
     else:
         return Ta('error-noUNC@1') % value, None
 
 
-def validate_safedir(root, value):
+def validate_safedir(root, value, default):
     """ Allow only when queues are empty and no UNC """
     if sabnzbd.empty_queues():
-        return validate_no_unc(root, value)
+        return validate_no_unc(root, value, default)
     else:
         return Ta('error-QnotEmpty'), None
 
 
-def validate_dir_exists(root, value):
+def validate_dir_exists(root, value, default):
     """ Check if directory exists """
     p = sabnzbd.misc.real_path(root, value)
     if os.path.exists(p):
@@ -874,6 +881,15 @@ def validate_dir_exists(root, value):
     else:
         return Ta('error-noFolder@1') % p, None
 
+
+def validate_notempty(root, value, default):
+    """ If value is empty, return default """
+    if value:
+        return None, value
+    else:
+        return None, default
+
+
 def create_api_key():
     import time
     try:
