@@ -194,7 +194,7 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
         f.close()
     except:
         logging.warning(Ta('warn-noRead@1'), path)
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
         return -2
 
 
@@ -221,7 +221,7 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
         if not keep: os.remove(path)
     except:
         logging.error(Ta('error-remove@1'), path)
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
         return 1
 
     return 0
@@ -265,9 +265,11 @@ class DirScanner(threading.Thread):
 
         self.shutdown = False
         self.error_reported = False # Prevents mulitple reporting of missing watched folder
+        self.trigger = False
         self.dirscan_dir = cfg.DIRSCAN_DIR.get_path()
         self.dirscan_speed = cfg.DIRSCAN_SPEED.get()
         cfg.DIRSCAN_DIR.callback(self.newdir)
+        cfg.DIRSCAN_SPEED.callback(self.newspeed)
 
     def newdir(self):
         """ We're notified of a dir change """
@@ -276,6 +278,11 @@ class DirScanner(threading.Thread):
         self.dirscan_dir = cfg.DIRSCAN_DIR.get_path()
         self.dirscan_speed = cfg.DIRSCAN_SPEED.get()
 
+    def newspeed(self):
+        """ We're notified of a dir change """
+        self.dirscan_speed = cfg.DIRSCAN_SPEED.get()
+        self.trigger = True
+
     def stop(self):
         self.save()
         logging.info('Dirscanner shutting down')
@@ -371,11 +378,12 @@ class DirScanner(threading.Thread):
             # Use variable scan delay
             dirscan_dir = self.dirscan_dir
             x = self.dirscan_speed
-            while (x > 0) and not self.shutdown:
+            while (x > 0) and not self.shutdown and not self.trigger:
                 time.sleep(1.0)
                 x = x - 1
 
             if dirscan_dir and not self.shutdown and not sabnzbd.PAUSED_ALL:
+                self.trigger = False
                 run_dir(dirscan_dir, None)
 
                 try:
