@@ -45,7 +45,7 @@ if sabnzbd.FOUNDATION:
     import Foundation
 
 RE_VERSION = re.compile('(\d+)\.(\d+)\.(\d+)([a-zA-Z]*)(\d*)')
-RE_UNITS = re.compile('(\d+\.*\d*)\s*([KMGTP]*)', re.I)
+RE_UNITS = re.compile('(\d+\.*\d*)\s*([KMGTP]{0,1})', re.I)
 TAB_UNITS = ('', 'K', 'M', 'G', 'T', 'P')
 
 PANIC_NONE  = 0
@@ -55,6 +55,7 @@ PANIC_QUEUE = 3
 PANIC_FWALL = 4
 PANIC_OTHER = 5
 PANIC_XPORT = 6
+PANIC_SQLITE = 7
 
 def safe_lower(txt):
     if txt:
@@ -100,6 +101,73 @@ def cat_to_opts(cat, pp=None, script=None, priority=None):
 
     return cat, pp, script, priority
 
+
+#------------------------------------------------------------------------------
+_wildcard_to_regex = {
+    '\\': r'\\',
+    '^' : r'\^',
+    '$' : r'\$',
+    '.' : r'\.',
+    '[' : r'\[',
+    ']' : r'\]',
+    '(' : r'\(',
+    ')' : r'\)',
+    '+' : r'\+',
+    '?' : r'.' ,
+    '|' : r'\|',
+    '{' : r'\{',
+    '}' : r'\}',
+    '*' : r'.*'
+}
+def wildcard_to_re(text):
+    """ Convert plain wildcard string (with '*' and '?') to regex.
+    """
+    return ''.join([_wildcard_to_regex.get(ch, ch) for ch in text])
+
+
+#------------------------------------------------------------------------------
+def CatConvert(cat):
+    """ Convert newzbin/nzbs.org category/group-name to user categories.
+        If no match found, but newzbin-cat equals user-cat, then return user-cat
+        If no match found, return None
+    """
+    newcat = cat
+    found = False
+
+    if cat and cat.lower() != 'none':
+        cats = config.get_categories()
+        for ucat in cats:
+            try:
+                newzbin = cats[ucat].newzbin.get()
+                if type(newzbin) != type([]):
+                    newzbin = [newzbin]
+            except:
+                newzbin = []
+            for name in newzbin:
+                if re.search('^%s$' % wildcard_to_re(name), cat, re.I):
+                    if '.' not in name:
+                        logging.debug('Convert index site category "%s" to user-cat "%s"', cat, ucat)
+                    else:
+                        logging.debug('Convert group "%s" to user-cat "%s"', cat, ucat)
+                    newcat = ucat
+                    found = True
+                    break
+            if found:
+                break
+
+        if not found:
+            for ucat in cats:
+                if cat.lower() == ucat.lower():
+                    found = True
+                    break
+
+    if found:
+        return newcat
+    else:
+        return None
+
+
+
 ################################################################################
 # sanitize_filename                                                            #
 ################################################################################
@@ -267,7 +335,7 @@ def get_user_shellfolders():
             name, value, val_type = _winreg.EnumValue(key, i)
             try:
                 values[name] = value.encode('latin-1')
-            except UnicodeDecodeError:
+            except UnicodeEncodeError:
                 try:
                     # If the path name cannot be converted to latin-1 (contains high ASCII value strings)
                     # then try and use the short name
@@ -278,6 +346,7 @@ def get_user_shellfolders():
                 except:
                     # probably a pywintypes.error error such as folder does not exist
                     logging.error("Traceback: ", exc_info = True)
+                    values[name] = 'c:\\'
             i += 1
         _winreg.CloseKey(key)
         _winreg.CloseKey(hive)
@@ -366,8 +435,8 @@ MSG_BAD_PORT = r'''
 
 MSG_ILL_PORT = r'''
     SABnzbd needs a free tcp/ip port for its internal web server.<br>
-    Port %s on %s was tried , but the account SABnzbd has no permission to use it.<br>
-    On Linux systems, normal users must use ports above 1023.<br>
+    Port %s on %s was tried , but the account used for SABnzbd has no permission to use it.<br>
+    On OSX and Linux systems, normal users must use ports above 1023.<br>
     <br>
     Please restart SABnzbd with a different port number.<br>
     <br>
@@ -410,6 +479,13 @@ MSG_OLD_QUEUE = r'''
     <FORM><input type="button" onclick="this.form.action='/.'; this.form.submit(); return false;" value="OK"/></FORM>
 '''
 
+MSG_SQLITE = r'''
+    SABnzbd detected that the file sqlite3.dll is missing.<br><br>
+    Some poorly designed virus-scanners remove this file.<br>
+    Please check your virus-scanner, try to re-install SABnzbd and complain to your virus-scanner vendor.<br>
+    <br>
+'''
+
 def panic_message(panic, a=None, b=None):
     """Create the panic message from templates
     """
@@ -443,6 +519,8 @@ def panic_message(panic, a=None, b=None):
             msg = MSG_BAD_FWALL % "It is likely that you are using ZoneAlarm on Vista.<br>"
         else:
             msg = MSG_BAD_FWALL % "<br>"
+    elif panic == PANIC_SQLITE:
+        msg = MSG_SQLITE
     else:
         msg = MSG_OTHER % (a, b)
 
@@ -471,6 +549,9 @@ def panic_queue(name):
 def panic_tmpl(name):
     launch_a_browser(panic_message(PANIC_TEMPL, name, 0))
 
+def panic_sqlite(name):
+    launch_a_browser(panic_message(PANIC_SQLITE, name, 0))
+
 def panic_old_queue():
     msg = MSG_OLD_QUEUE
     return MSG_BAD_NEWS % (sabnzbd.MY_NAME, sabnzbd.__version__, sabnzbd.MY_NAME, sabnzbd.__version__, msg, '')
@@ -741,7 +822,7 @@ def create_dirs(dirpath):
         logging.info('Creating directories: %s', dirpath)
         if not create_all_dirs(dirpath, True):
             logging.error(Ta('error-makeFile@1'), dirpath)
-            logging.debug("Traceback: ", exc_info = True)
+            logging.info("Traceback: ", exc_info = True)
             return None
     return dirpath
 
@@ -766,7 +847,7 @@ def move_to_path(path, new_path, unique=True):
                 os.remove(path)
             except:
                 logging.error(Ta('error-moveFile@2'), path, new_path)
-                logging.debug("Traceback: ", exc_info = True)
+                logging.info("Traceback: ", exc_info = True)
     return new_path
 
 
@@ -821,16 +902,20 @@ def get_filepath(path, nzo, filename):
     return fullPath
 
 
-def bad_fetch(nzo, url, msg='', retry=False, archive=False):
-    """ Create History entry for failed URL Fetch """
+def bad_fetch(nzo, url, msg='', retry=False, content=False):
+    """ Create History entry for failed URL Fetch
+        msg : message to be logged
+        retry : make retry link in histort
+        content : report in history that cause is a bad NZB file
+    """
     logging.error(Ta('error-urlGet@2'), latin1(url), latin1(msg))
     msg = unicoder(msg)
 
     pp = nzo.get_pp()
-    if pp:
-        pp = '&pp=%s' % urllib.quote(pp)
-    else:
+    if pp is None:
         pp = ''
+    else:
+        pp = '&pp=%s' % str(pp)
     cat = nzo.get_cat()
     if cat:
         cat = '&cat=%s' % urllib.quote(cat)
@@ -849,22 +934,23 @@ def bad_fetch(nzo, url, msg='', retry=False, archive=False):
         nzo.set_filename(url)
         nzo.set_original_dirname(url)
 
+    if content:
+        # Bad content
+        msg = T('his-badArchive')
+    else:
+        # Failed fetch
+        msg = ' (' + msg + ')'
+
     if retry:
         nzbname = nzo.get_dirname_rename()
         if nzbname:
             nzbname = '&nzbname=%s' % urllib.quote(nzbname)
         else:
             nzbname = ''
-        text = T('his-retryURL1@1')+', <a href="./retry?session=%s&url=%s%s%s%s%s">' + T('his-retryURL2') + '</a>'
+        text = T('his-retryURL1@1') + ', <a href="./retry?session=%s&url=%s%s%s%s%s">' + T('his-retryURL2') + '</a>'
         parms = (msg, cfg.API_KEY.get(), urllib.quote(url), pp, cat, script, nzbname)
         nzo.set_fail_msg(text % parms)
     else:
-        if archive:
-            msg = T('his-badArchive')
-        elif not '://' in url:
-            msg = T('his-cannotGetReport')
-        else:
-            msg = T('his-failedURL')
         nzo.set_fail_msg(msg)
 
     sabnzbd.nzbqueue.remove_nzo(nzo.nzo_id, add_to_history=True, unload=True)
@@ -901,16 +987,10 @@ def loadavg():
     if sabnzbd.WIN32 or sabnzbd.DARWIN:
         return ""
     try:
-        loadavgstr = open('/proc/loadavg', 'r').readline().strip()
+        return "%.2f, %.2f, %.2f" % os.getloadavg()
     except:
         return ""
 
-    data = loadavgstr.split()
-    try:
-        a1, a5, a15 = map(float, data[:3])
-        return "%.2f, %.2f, %.2f" % (a1, a5, a15)
-    except:
-        return ""
 
 
 def format_time_string(seconds, days=0):
@@ -1017,7 +1097,7 @@ def create_https_certificates(ssl_cert, ssl_key):
         open(ssl_cert, 'w').write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
     except:
         logging.error(Ta('error-sslFiles'))
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
         return False
 
     return True
@@ -1087,7 +1167,7 @@ def win_hibernate():
         time.sleep(10)
     except:
         logging.error(Ta('error-hibernate'))
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
 
 
 def win_standby():
@@ -1096,7 +1176,7 @@ def win_standby():
         time.sleep(10)
     except:
         logging.error(Ta('error-standby'))
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
 
 
 def win_shutdown():
@@ -1123,7 +1203,7 @@ def osx_shutdown():
         subprocess.call(['osascript', '-e', 'tell app "System Events" to shut down'])
     except:
         logging.error(Ta('error-shutdown'))
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
     os._exit(0)
 
 
@@ -1133,7 +1213,7 @@ def osx_standby():
         time.sleep(10)
     except:
         logging.error(Ta('error-standby'))
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
 
 
 def osx_hibernate():
@@ -1251,7 +1331,7 @@ def renamer(old, new):
                 return
             except WindowsError, err:
                 if err[0] == 32:
-                    logging.info('Retry rename %s to %s', old, new)
+                    logging.debug('Retry rename %s to %s', old, new)
                     retries -= 1
                 else:
                     raise WindowsError(err)
@@ -1271,7 +1351,7 @@ def remove_dir(path):
                 return
             except WindowsError, err:
                 if err[0] == 32:
-                    logging.info('Retry delete %s', path)
+                    logging.debug('Retry delete %s', path)
                     retries -= 1
                 else:
                     raise WindowsError(err)
