@@ -49,8 +49,8 @@ else:
             return repr(self.parameter)
 
 # Regex globals
-RAR_RE = re.compile(r'\.(?P<ext>part\d*\.rar|rar|s\d\d|r\d\d|\d\d\d)$', re.I)
-RAR_RE_V3 = re.compile(r'\.(?P<ext>part\d*)$', re.I)
+RAR_RE = re.compile(r'\.part\d+\.rar|rar|[r-z]\d\d|\d\d\d$', re.I)
+RAR_RE_V3 = re.compile(r'\.part\d*$', re.I)
 
 LOADING_RE = re.compile(r'^Loading "(.+)"')
 TARGET_RE = re.compile(r'^(?:File|Target): "(.+)" -')
@@ -93,7 +93,7 @@ def find_programs(curdir):
         sabnzbd.newsunpack.RAR_COMMAND =  check(curdir, 'osx/unrar/unrar')
 
     if sabnzbd.WIN32:
-        if sabnzbd.WIN64:
+        if sabnzbd.WIN64 and cfg.allow_64bit_tools.get():
             sabnzbd.newsunpack.PAR2_COMMAND =  check(curdir, 'win/par2/x64/par2.exe')
             sabnzbd.newsunpack.RAR_COMMAND =   check(curdir, 'win/unrar/x64/UnRAR.exe')
         if not sabnzbd.newsunpack.PAR2_COMMAND:
@@ -112,8 +112,7 @@ def find_programs(curdir):
         sabnzbd.newsunpack.ZIP_COMMAND = find_on_path('unzip')
 
     if not (sabnzbd.WIN32 or sabnzbd.DARWIN):
-        if not cfg.ignore_wrong_unrar.get():
-            sabnzbd.newsunpack.RAR_PROBLEM = not unrar_check(sabnzbd.newsunpack.RAR_COMMAND)
+        sabnzbd.newsunpack.RAR_PROBLEM = not unrar_check(sabnzbd.newsunpack.RAR_COMMAND)
 
 #------------------------------------------------------------------------------
 def external_processing(extern_proc, complete_dir, filename, msgid, nicename, cat, group, status):
@@ -122,6 +121,7 @@ def external_processing(extern_proc, complete_dir, filename, msgid, nicename, ca
                str(nicename), str(msgid), str(cat), str(group), str(status)]
 
     stup, need_shell, command, creationflags = build_command(command)
+    env = fix_env()
 
     logging.info('Running external script %s(%s, %s, %s, %s, %s, %s, %s)', \
                  extern_proc, complete_dir, filename, nicename, msgid, cat, group, status)
@@ -129,9 +129,9 @@ def external_processing(extern_proc, complete_dir, filename, msgid, nicename, ca
     try:
         p = subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                            startupinfo=stup, creationflags=creationflags)
+                            startupinfo=stup, env=env, creationflags=creationflags)
     except:
-        logging.debug("Failed script %s, Traceback: ", extern_proc, exc_info = True)
+        logging.info("Failed script %s, Traceback: ", extern_proc, exc_info = True)
         return "Cannot run script %s\r\n" % extern_proc, -1
 
     output = p.stdout.read()
@@ -155,8 +155,12 @@ def SimpleRarExtract(rarfile, name):
     return ret, output
 
 #------------------------------------------------------------------------------
+def unpack_magic(nzo, workdir, workdir_complete, dele, joinables, zips, rars, ts, depth=0):
+    if depth > 5:
+        logging.warning('Unpack nesting too deep [%s]', nzo.get_dirname())
+        return False, []
+    depth += 1
 
-def unpack_magic(nzo, workdir, workdir_complete, dele, joinables, zips, rars, ts):
     xjoinables, xzips, xrars, xts = build_filelists(workdir, workdir_complete)
 
     rerun = False
@@ -164,63 +168,43 @@ def unpack_magic(nzo, workdir, workdir_complete, dele, joinables, zips, rars, ts
     error = False
 
     if cfg.enable_filejoin.get():
-        do_filejoin = False
-        for joinable in xjoinables:
-            if joinable not in joinables:
-                do_filejoin = True
-                rerun = True
-                break
-
-        if do_filejoin:
+        new_joins = [jn for jn in xjoinables if jn not in joinables]
+        if new_joins:
+            rerun = True
             logging.info('Filejoin starting on %s', workdir)
-            error, newf = file_join(nzo, workdir, workdir_complete, dele, xjoinables)
+            error, newf = file_join(nzo, workdir, workdir_complete, dele, new_joins)
             if newf:
                 newfiles.extend(newf)
             logging.info('Filejoin finished on %s', workdir)
             nzo.set_action_line('', '')
 
     if cfg.enable_unrar.get():
-        do_unrar = False
-        for rar in xrars:
-            if rar not in rars:
-                do_unrar = True
-                rerun = True
-                break
-
-        if do_unrar:
+        new_rars = [rar for rar in xrars if rar not in rars]
+        if new_rars:
+            rerun = True
             logging.info('Unrar starting on %s', workdir)
-            error, newf = rar_unpack(nzo, workdir, workdir_complete, dele, xrars)
+            error, newf = rar_unpack(nzo, workdir, workdir_complete, dele, new_rars)
             if newf:
                 newfiles.extend(newf)
             logging.info('Unrar finished on %s', workdir)
             nzo.set_action_line('', '')
 
     if cfg.enable_unzip.get():
-        do_unzip = False
-        for _zip in xzips:
-            if _zip not in zips:
-                do_unzip = True
-                rerun = True
-                break
-
-        if do_unzip:
+        new_zips = [zip for zip in xzips if zip not in zips]
+        if new_zips:
+            rerun = True
             logging.info('Unzip starting on %s', workdir)
-            if unzip(nzo, workdir, workdir_complete, dele, xzips):
+            if unzip(nzo, workdir, workdir_complete, dele, new_zips):
                 error = True
             logging.info('Unzip finished on %s', workdir)
             nzo.set_action_line('', '')
 
     if cfg.enable_tsjoin.get():
-        do_tsjoin = False
-        for _ts in xts:
-            if _ts not in ts:
-                do_tsjoin = True
-                rerun = True
-                break
-
-        if do_tsjoin:
+        new_ts = [_ts for _ts in xts if _ts not in ts]
+        if new_ts:
+            rerun = True
             logging.info('TS Joining starting on %s', workdir)
-            error, newf = file_join(nzo, workdir, workdir_complete, dele, xts)
+            error, newf = file_join(nzo, workdir, workdir_complete, dele, new_ts)
             if newf:
                 newfiles.extend(newf)
             logging.info('TS Joining finished on %s', workdir)
@@ -229,7 +213,7 @@ def unpack_magic(nzo, workdir, workdir_complete, dele, joinables, zips, rars, ts
 
     if rerun:
         z, y = unpack_magic(nzo, workdir, workdir_complete, dele, xjoinables,
-                            xzips, xrars, xts)
+                            xzips, xrars, xts, depth)
         if z:
             error = z
         if y:
@@ -391,10 +375,10 @@ def rar_unpack(nzo, workdir, workdir_complete, delete, rars):
 
         rarpath = rar_sets[rar_set][0]
 
-
-        extraction_path = workdir
-        if workdir_complete:
+        if workdir_complete and rarpath.startswith(workdir):
             extraction_path = workdir_complete
+        else:
+            extraction_path = os.path.split(rarpath)[0]
 
         logging.info("Extracting rarfile %s (belonging to %s) to %s",
                      rarpath, rar_set, extraction_path)
@@ -402,10 +386,9 @@ def rar_unpack(nzo, workdir, workdir_complete, delete, rars):
         try:
             newfiles, rars = RAR_Extract(rarpath, len(rar_sets[rar_set]),
                                          nzo, rar_set, extraction_path)
-            success = True
+            success = newfiles and rars
         except:
             success = False
-            errors = True
             msg = sys.exc_info()[1]
             nzo.set_fail_msg(T('error-unpackFail@1') % msg)
             setname = nzo.get_dirname()
@@ -436,7 +419,7 @@ def rar_unpack(nzo, workdir, workdir_complete, delete, rars):
                     except OSError:
                         logging.warning(Ta('warn-delFailed@1'), brokenrar)
 
-    return errors, extracted_files
+    return not success, extracted_files
 
 
 def RAR_Extract(rarfile, numrars, nzo, setname, extraction_path):
@@ -450,11 +433,8 @@ def RAR_Extract(rarfile, numrars, nzo, setname, extraction_path):
         expected_files = zf.unamelist()
         zf.close()
     except:
-        nzo.set_fail_msg(T('error-badArchive'))
-        nzo.set_unpack_info('Unpack', u'[%s] %s' % (unicoder(setname), T('error-badArchive')), set=setname)
-
-        logging.info('Archive %s probably encrypted, skipping', rarfile)
-        return ((), ())
+        logging.info('Archive %s possibly encrypted, will skip content check', rarfile)
+        expected_files = []
 
     if nzo.get_password():
         password = '-p%s' % nzo.get_password()
@@ -464,9 +444,15 @@ def RAR_Extract(rarfile, numrars, nzo, setname, extraction_path):
     ############################################################################
 
     if sabnzbd.WIN32:
+        # Use all flags
         command = ['%s' % RAR_COMMAND, 'x', '-idp', '-o-', '-or', '-ai', password,
                    '%s' % rarfile, '%s/' % extraction_path]
+    elif RAR_PROBLEM:
+        # Use only oldest options (specifically no "-or")
+        command = ['%s' % RAR_COMMAND, 'x', '-idp', '-o-', password,
+                   '%s' % rarfile, '%s/' % extraction_path]
     else:
+        # Don't use "-ai" (not needed for non-Windows)
         command = ['%s' % RAR_COMMAND, 'x', '-idp', '-o-', '-or', password,
                    '%s' % rarfile, '%s/' % extraction_path]
 
@@ -557,8 +543,8 @@ def RAR_Extract(rarfile, numrars, nzo, setname, extraction_path):
     p.wait()
 
 
-    if cfg.UNPACK_CHECK.get():
-        if reliable_unpack_names():
+    if cfg.UNPACK_CHECK.get() and expected_files:
+        if reliable_unpack_names() and not RAR_PROBLEM:
             all_found = True
             for path in expected_files:
                 path = unicode2local(path)
@@ -574,7 +560,7 @@ def RAR_Extract(rarfile, numrars, nzo, setname, extraction_path):
                 nzo.set_unpack_info('Unpack', T('error-unpackMissing'), set=setname)
                 return ((), ())
         else:
-            logging.info('Skipping unrar file check due to unreliable file names')
+            logging.info('Skipping unrar file check due to unreliable file names or old unrar')
 
     msg = T('msg-unpackDone@2') % (str(len(extracted)), format_time_string(time() - start))
     nzo.set_unpack_info('Unpack', '[%s] %s' % (unicoder(setname), msg), set=setname)
@@ -596,9 +582,10 @@ def unzip(nzo, workdir, workdir_complete, delete, zips):
             logging.info("Starting extract on zipfile: %s ", _zip)
             nzo.set_action_line(T('msg-unpacking'), '%s' % unicoder(_zip))
 
-            extraction_path = workdir
-            if workdir_complete:
+            if workdir_complete and _zip.startswith(workdir):
                 extraction_path = workdir_complete
+            else:
+                extraction_path = os.path.split(_zip)[0]
 
             if ZIP_Extract(_zip, extraction_path):
                 unzip_failed = True
@@ -708,7 +695,7 @@ def par2_repair(parfile_nzf, nzo, workdir, setname):
             msg = sys.exc_info()[1]
             nzo.set_fail_msg(T('error-repairFailed@1') % msg)
             logging.error(Ta('error-filePar2@2'), msg, setname)
-            logging.debug("Traceback: ", exc_info = True)
+            logging.info("Traceback: ", exc_info = True)
             return readd, result
 
     try:
@@ -1023,6 +1010,21 @@ def PAR_Verify(parfile, parfile_nzf, nzo, setname, joinables, classic=False):
 
 #-------------------------------------------------------------------------------
 
+_RE_PYTHON = re.compile(r'^#!(.*/python)\s+(.*)$')
+
+def fix_env():
+    """ OSX: Return copy of environment without PYTHONPATH and PYTHONHOME
+        other: return None
+    """
+    if sabnzbd.DARWIN:
+        env = os.environ.copy()
+        if 'PYTHONPATH' in env: del env['PYTHONPATH']
+        if 'PYTHONHOME' in env: del env['PYTHONHOME']
+        return env
+    else:
+        return None
+
+
 def build_command(command):
     if not sabnzbd.WIN32:
         if IONICE_COMMAND and cfg.ionice.get().strip():
@@ -1042,7 +1044,7 @@ def build_command(command):
         creationflags = 0
 
     else:
-        need_shell = os.path.splitext(command[0])[1].lower() not in ('.exe', '.com', '.bat', '.cmd')
+        need_shell = os.path.splitext(command[0])[1].lower() not in ('.exe', '.com')
         stup = subprocess.STARTUPINFO()
         stup.dwFlags = STARTF_USESHOWWINDOW
         stup.wShowWindow = SW_HIDE
@@ -1052,7 +1054,8 @@ def build_command(command):
         # scripts with spaces in the path don't work.
         if need_shell and ' ' in command[0]:
             command[0] = win32api.GetShortPathName(command[0])
-
+        if need_shell:
+            command = list2cmdline(command)
     return (stup, need_shell, command, creationflags)
 
 # Sort the various RAR filename formats properly :\
@@ -1081,20 +1084,23 @@ def par_sort(a, b):
     elif bext == 'par2':
         return 1
 
+
 def build_filelists(workdir, workdir_complete, check_rar=True):
-    joinables, zips, rars = ([], [], [])
-
-    filelist = []
-
-    for root, dirs, files in os.walk(workdir):
-        for _file in files:
-            filelist.append(os.path.join(root, _file))
+    """ Build filelists, if workdir_complete has files, ignore workdir.
+        Optionally test content to establish RAR-ness
+    """
+    joinables, zips, rars, filelist = ([], [], [], [])
 
     if workdir_complete:
         for root, dirs, files in os.walk(workdir_complete):
             for _file in files:
                 filelist.append(os.path.join(root, _file))
 
+    if workdir and not filelist:
+        for root, dirs, files in os.walk(workdir):
+            for _file in files:
+                filelist.append(os.path.join(root, _file))
+
     if check_rar:
         joinables = [f for f in filelist if SPLITFILE_RE.search(f) and not israr(f)]
     else:
@@ -1203,3 +1209,15 @@ def unrar_check(rar):
         if m:
             return (int(m.group(1)), int(m.group(2))) >= (3, 80)
     return False
+
+
+#------------------------------------------------------------------------------
+def list2cmdline(lst):
+    """ convert list to a cmd.exe-compatible command string """
+    nlst = []
+    for arg in lst:
+        if (' ' in arg) or ('\t' in arg) or ('&' in arg) or ('|' in arg):
+            nlst.append('"%s"' % arg)
+        else:
+            nlst.append(arg)
+    return ' '.join(nlst)
