@@ -37,8 +37,7 @@ from threading import *
 import sabnzbd
 from sabnzbd.constants import *
 from sabnzbd.decorators import synchronized
-from sabnzbd.misc import cat_to_opts, sanitize_foldername, bad_fetch
-from sabnzbd.nzbstuff import CatConvert
+from sabnzbd.misc import cat_to_opts, sanitize_foldername, bad_fetch, CatConvert
 from sabnzbd.codecs import name_fixer
 import sabnzbd.newswrapper
 import sabnzbd.nzbqueue
@@ -78,7 +77,7 @@ def getBookmarksList():
 
 def delete_bookmark(msgid):
     global __BOOKMARKS
-    if __BOOKMARKS and cfg.NEWZBIN_BOOKMARKS.get() and cfg.NEWZBIN_UNBOOKMARK.get():
+    if __BOOKMARKS and cfg.NEWZBIN_UNBOOKMARK.get():
         __BOOKMARKS.del_bookmark(msgid)
 
 
@@ -202,7 +201,7 @@ class MSGIDGrabber(Thread):
                     sabnzbd.nzbqueue.insert_future_nzo(nzo, filename, msgid, data, pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname, nzo_info=nzo_info)
                 except:
                     logging.error(Ta('error-nbUpdate@1'), msgid)
-                    logging.debug("Traceback: ", exc_info = True)
+                    logging.info("Traceback: ", exc_info = True)
                     sabnzbd.nzbqueue.remove_nzo(nzo.nzo_id, False)
                 msgid = None
             else:
@@ -210,7 +209,7 @@ class MSGIDGrabber(Thread):
                     sleeper(int(filename))
                 else:
                     # Fatal error, give up on this one
-                    bad_fetch(nzo, msgid, retry=False)
+                    bad_fetch(nzo, msgid, msg=nzo_info, retry=True)
                     msgid = None
 
             osx.sendGrowlMsg(T('grwl-nzbadd-title'),filename,osx.NOTIFICATION['download'])
@@ -224,7 +223,7 @@ class MSGIDGrabber(Thread):
 def _grabnzb(msgid):
     """ Grab one msgid from newzbin """
 
-    nothing  = (None, None, None, None)
+    msg = ''
     retry = (60, None, None, None)
     nzo_info = {'msgid': msgid}
 
@@ -234,7 +233,7 @@ def _grabnzb(msgid):
 
     # Connect to Newzbin
     try:
-        if _HAVE_SSL:
+        if _HAVE_SSL and cfg.NEWZBIN_HTTPS.get():
             conn = httplib.HTTPSConnection('www.newzbin.com')
         else:
             conn = httplib.HTTPConnection('www.newzbin.com')
@@ -253,7 +252,7 @@ def _grabnzb(msgid):
 
     except:
         _warn_user('Problem accessing Newzbin server, wait 1 min.')
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
         return retry
 
     # Get the filename
@@ -266,7 +265,7 @@ def _grabnzb(msgid):
         pass
     if not (rcode or rtext):
         logging.error(T('error-nbProtocol'))
-        return nothing
+        return None, None, None, None
 
     # Official return codes:
     # 200 = OK, NZB content follows
@@ -295,31 +294,31 @@ def _grabnzb(msgid):
             wait = 60
         if wait > 60:
             wait = 60
-        logging.info("Newzbin says we should wait for %s sec", wait)
+        logging.debug("Newzbin says we should wait for %s sec", wait)
         return int(wait+1), None, None, None
 
     if rcode in ('402'):
-        logging.warning(Ta('warn-nbCredit'))
-        return nothing
+        msg = Ta('warn-nbCredit')
+        return None, None, None, msg
 
     if rcode in ('401'):
-        logging.warning(Ta('warn-nbNoAuth'))
-        return nothing
+        msg = Ta('warn-nbNoAuth')
+        return None, None, None, msg
 
     if rcode in ('400', '404'):
-        logging.error(Ta('error-nbReport@1'), msgid)
-        return nothing
+        msg = Ta('error-nbReport@1') % msgid
+        return None, None, None, msg
 
     if rcode != '200':
-        logging.error(Ta('error-nbUnkownError@2'), rcode, rtext)
-        return nothing
+        msg = Ta('error-nbUnkownError@2') % (rcode, rtext)
+        return None, None, None, msg
 
     # Process data
     report_name = response.getheader('X-DNZB-Name')
     report_cat  = response.getheader('X-DNZB-Category')
     if not (report_name and report_cat):
-        logging.error(Ta('error-nbInfo@1'), msgid)
-        return nothing
+        msg = Ta('error-nbInfo@1') %  msgid
+        return None, None, None, msg
 
     # sanitize report_name
     newname = sanitize_foldername(report_name)
@@ -353,7 +352,7 @@ class Bookmarks:
 
         # Connect to Newzbin
         try:
-            if _HAVE_SSL:
+            if _HAVE_SSL and cfg.NEWZBIN_HTTPS.get():
                 conn = httplib.HTTPSConnection('www.newzbin.com')
             else:
                 conn = httplib.HTTPConnection('www.newzbin.com')
@@ -374,7 +373,7 @@ class Bookmarks:
             response = conn.getresponse()
         except:
             _warn_user('Problem accessing Newzbin server.')
-            logging.debug("Traceback: ", exc_info = True)
+            logging.info("Traceback: ", exc_info = True)
             return
 
         data = response.read()
@@ -420,7 +419,7 @@ class Bookmarks:
                     if msgid and (msgid not in self.bookmarks):
                         self.bookmarks.append(msgid)
                         logging.info("Found new bookmarked msgid %s (%s)", msgid, text)
-                        sabnzbd.add_msgid(int(msgid), None, None, priority=cfg.DIRSCAN_PRIORITY.get())
+                        sabnzbd.add_msgid(int(msgid), None, None, priority=None)
         else:
             logging.error(Ta('error-nbUnkownError@1'), rcode)
 
