@@ -23,7 +23,6 @@ import os
 import time
 import re
 import logging
-import sabnzbd
 import datetime
 import xml.sax
 import xml.sax.handler
@@ -40,8 +39,8 @@ import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.trylist import TryList
 from sabnzbd.lang import T, Ta
-from sabnzbd.misc import to_units, cat_to_opts
-from sabnzbd.codecs import unicoder
+from sabnzbd.misc import to_units, cat_to_opts, CatConvert
+from sabnzbd.codecs import unicoder, platform_encode
 
 RE_NEWZBIN = re.compile(r"msgid_(\w+) (.+)(\.nzb)$", re.I)
 RE_NORMAL  = re.compile(r"(.+)(\.nzb)", re.I)
@@ -149,7 +148,7 @@ class NzbFile(TryList):
         self.__article_count += 1
 
     def finish_import(self):
-        logging.info("Finishing import on %s", self.__subject)
+        logging.debug("Finishing import on %s", self.__subject)
 
         article_db = sabnzbd.load_data(self.nzf_id)
         if article_db:
@@ -301,6 +300,7 @@ class NzbParser(xml.sax.handler.ContentHandler):
         self.valids = 0
         self.skipped_files = 0
         self.nzf_list = []
+        self.groups = []
 
     def startDocument(self):
         self.filter = cfg.IGNORE_SAMPLES.get()
@@ -348,7 +348,6 @@ class NzbParser(xml.sax.handler.ContentHandler):
 
         elif name == 'groups' and self.in_nzb and self.in_file:
             self.in_groups = True
-            self.groups = []
 
         elif name == 'nzb':
             self.in_nzb = True
@@ -361,7 +360,9 @@ class NzbParser(xml.sax.handler.ContentHandler):
 
     def endElement(self, name):
         if name == 'group' and self.in_group:
-            self.groups.append(str(''.join(self.group_name)))
+            group = str(''.join(self.group_name))
+            if group not in self.groups:
+                self.groups.append(group)
             self.in_group = False
 
         elif name == 'segment' and self.in_segment:
@@ -411,7 +412,8 @@ class NzbParser(xml.sax.handler.ContentHandler):
     def endDocument(self):
         """ End of the file """
         self.nzo._NzbObject__group = self.groups
-        self.nzo._NzbObject__avg_date = datetime.datetime.fromtimestamp(self.avg_age / self.valids)
+        files = max(1, self.valids)
+        self.nzo._NzbObject__avg_date = datetime.datetime.fromtimestamp(self.avg_age / files)
         if self.skipped_files:
             logging.warning(Ta('warn-badImport@2'),
                             self.skipped_files, self.nzo.get_filename())
@@ -432,6 +434,9 @@ class NzbObject(TryList):
                  priority=NORMAL_PRIORITY, nzbname=None, status="Queued", nzo_info=None):
         TryList.__init__(self)
 
+        filename = platform_encode(filename)
+        nzbname = platform_encode(nzbname)
+
         if pp is None:
             r = u = d = None
         else:
@@ -593,6 +598,14 @@ class NzbObject(TryList):
         # Set nzo save-delay to 6 sec per GB with a max of 5 min
         self.extra4 = min(6.0 * float(self.__bytes) / GIGI, 300.0)
 
+        # Pause job when above size limit
+        limit = cfg.SIZE_LIMIT.get_int()
+        if limit and self.__bytes > limit:
+            logging.info('Job too large, forcing low prio and paused (%s)', self.__dirname)
+            self.pause_nzo()
+            self.set_priority(LOW_PRIORITY)
+
+
     ## begin nzo.Mutators #####################################################
     ## excluding nzo.__try_list ###############################################
     def check_for_dupe(self, nzf):
@@ -723,14 +736,11 @@ class NzbObject(TryList):
         return self.extra2
 
     def set_original_dirname(self, name):
-        if isinstance(name, str):
-            self.__original_dirname = name.strip()
-        else:
-            self.__original_dirname = name
+        self.__original_dirname = platform_encode(name.strip())
 
     def set_name(self, name):
         if isinstance(name, str):
-            name, self.extra2 = scan_password(name)
+            name, self.extra2 = scan_password(platform_encode(name))
             self.__original_dirname = sabnzbd.misc.sanitize_foldername(name)
             return True
         return False
@@ -808,11 +818,16 @@ class NzbObject(TryList):
             # Don't try to get an article if server is in try_list of nzf
             if not nzf.server_in_try_list(server):
                 if not nzf.import_finished:
-                    nzf.finish_import()
-                    # Still not finished? Something went wrong...
-                    if not nzf.import_finished:
-                        logging.error(Ta('error-qImport@1'), nzf)
-                        nzf_remove_list.append(nzf)
+                    # Only load NZF when it's a primary server
+                    # or when it's a backup server without active primaries
+                    if server.fillserver ^ sabnzbd.active_primaries():
+                        nzf.finish_import()
+                        # Still not finished? Something went wrong...
+                        if not nzf.import_finished:
+                            logging.error(Ta('error-qImport@1'), nzf)
+                            nzf_remove_list.append(nzf)
+                            continue
+                    else:
                         continue
 
                 article = nzf.get_article(server)
@@ -822,12 +837,10 @@ class NzbObject(TryList):
         for nzf in nzf_remove_list:
             self.__files.remove(nzf)
 
-        if article:
-            return article
-        else:
+        if not article:
             # No articles for this server, block for next time
             self.add_to_try_list(server)
-            return
+        return article
 
     def move_top_bulk(self, nzf_ids):
         self.__cleanup_nzf_ids(nzf_ids)
@@ -1191,45 +1204,6 @@ def SplitFileName(name):
     else:
         return name.strip(), ""
 
-def CatConvert(cat):
-    """ Convert newzbin-category/group-name to user categories.
-        If no match found, but newzbin-cat equals user-cat, then return user-cat
-        If no match found, return None
-    """
-    newcat = cat
-    found = False
-
-    if cat and cat.lower() != 'none':
-        cats = config.get_categories()
-        for ucat in cats:
-            try:
-                newzbin = cats[ucat].newzbin.get()
-                if type(newzbin) != type([]):
-                    newzbin = [newzbin]
-            except:
-                newzbin = []
-            for name in newzbin:
-                if name.lower() == cat.lower():
-                    if '.' not in name:
-                        logging.debug('Convert newzbin-cat "%s" to user-cat "%s"', cat, ucat)
-                    else:
-                        logging.debug('Convert group "%s" to user-cat "%s"', cat, ucat)
-                    newcat = ucat
-                    found = True
-                    break
-            if found:
-                break
-
-        if not found:
-            for ucat in cats:
-                if cat.lower() == ucat.lower():
-                    found = True
-                    break
-
-    if found:
-        return newcat
-    else:
-        return None
 
 def format_time_string(seconds, days=0):
 
