@@ -38,7 +38,7 @@ from sabnzbd.constants import TOP_PRIORITY, POSTPROC_QUEUE_FILE_NAME, \
      POSTPROC_QUEUE_VERSION, sample_match
 from sabnzbd.codecs import TRANS, unicoder
 import sabnzbd.newzbin
-import sabnzbd.email as email
+import sabnzbd.emailer as emailer
 import sabnzbd.dirscanner as dirscanner
 import sabnzbd.downloader as downloader
 import sabnzbd.config as config
@@ -127,6 +127,7 @@ class PostProcessor(Thread):
             for nzo in self.history_queue:
                 self.process(nzo)
 
+        self.__busy = False # True while a job is being processed
 
     def save(self):
         """ Save postproc queue """
@@ -171,13 +172,14 @@ class PostProcessor(Thread):
         self.save()
 
     def empty(self):
-        return self.queue.empty()
+        return self.queue.empty() and not self.__busy
 
     def get_queue(self):
         return self.history_queue
 
     def run(self):
         while 1:
+            self.__busy = False
             if self.queue.empty(): HandleEmptyQueue()
 
             ## Get a job from the queue, quit on empty job
@@ -188,6 +190,7 @@ class PostProcessor(Thread):
             if cfg.PAUSE_ON_POST_PROCESSING.get():
                 downloader.idle_downloader()
 
+            self.__busy = True
             start = time.time()
 
             folder_rename = cfg.FOLDER_RENAME.get()
@@ -331,7 +334,7 @@ class PostProcessor(Thread):
                         remove_dir(workdir)
                 except:
                     logging.error(Ta('error-ppDelWorkdir@1'), workdir)
-                    logging.debug("Traceback: ", exc_info = True)
+                    logging.info("Traceback: ", exc_info = True)
 
 
                 ## Set permissions right
@@ -364,7 +367,7 @@ class PostProcessor(Thread):
                             nzo.set_dirname(os.path.basename(workdir_complete))
                         except:
                             logging.error(Ta('error-ppRename@2'), tmp_workdir_complete, workdir_complete)
-                            logging.debug("Traceback: ", exc_info = True)
+                            logging.info("Traceback: ", exc_info = True)
 
                     if unpackError: jobResult = jobResult + 2
 
@@ -402,7 +405,7 @@ class PostProcessor(Thread):
                     ## Email the results
                     if (not nzb_list) and cfg.EMAIL_ENDJOB.get():
                         if (cfg.EMAIL_ENDJOB.get() == 1) or (cfg.EMAIL_ENDJOB.get() == 2 and (unpackError or not parResult)):
-                            email.endjob(dirname, msgid, cat, mailResult, workdir_complete, nzo.get_bytes_downloaded(),
+                            emailer.endjob(dirname, msgid, cat, mailResult, workdir_complete, nzo.get_bytes_downloaded(),
                                          nzo.get_unpack_info(), script, TRANS(script_log), script_ret)
 
                     if fname:
@@ -434,7 +437,7 @@ class PostProcessor(Thread):
                 logging.error(Ta('error-ppFailed@1'), filename)
                 #except:
                 #    pass
-                logging.debug("Traceback: ", exc_info = True)
+                logging.info("Traceback: ", exc_info = True)
                 nzo.set_fail_msg(T('warn-PostCrash'))
                 osx.sendGrowlMsg("Download Failed",filename,osx.NOTIFICATION['complete'])
                 nzo.set_status('Failed')
@@ -464,7 +467,7 @@ class PostProcessor(Thread):
                 sabnzbd.nzbqueue.cleanup_nzo(nzo)
             except:
                 logging.error(Ta('error-ppCleanup@1'), nzo.get_dirname())
-                logging.debug("Traceback: ", exc_info = True)
+                logging.info("Traceback: ", exc_info = True)
 
             # Remove the nzo from the history_queue list
             # This list is simply used for the creation of the history in interface.py
@@ -486,7 +489,7 @@ def MakeLogFile(name, content):
         f = open(path, "w")
     except:
         logging.error(Ta('error-ppLogFile@1'), path)
-        logging.debug("Traceback: ", exc_info = True)
+        logging.info("Traceback: ", exc_info = True)
         return "a"
     f.write(content)
     f.close()
@@ -518,14 +521,14 @@ def perm_script(wdir, umask):
         except:
             if report_errors:
                 logging.error(Ta('error-ppPermissions@1'), root)
-                logging.debug("Traceback: ", exc_info = True)
+                logging.info("Traceback: ", exc_info = True)
         for name in files:
             try:
                 os.chmod(join(root, name), umask_file)
             except:
                 if report_errors:
                     logging.error(Ta('error-ppPermissions@1'), join(root, name))
-                    logging.debug("Traceback: ", exc_info = True)
+                    logging.info("Traceback: ", exc_info = True)
 
 
 def Cat2Dir(cat, defdir):
@@ -561,9 +564,8 @@ def addPrefixes(path,nzo):
 
 def HandleEmptyQueue():
     """ Check if empty queue calls for action """
-    sabnzbd.save_state()
-
     if sabnzbd.QUEUECOMPLETEACTION_GO:
+        sabnzbd.save_state()
         logging.info("Queue has finished, launching: %s (%s)", \
             sabnzbd.QUEUECOMPLETEACTION, sabnzbd.QUEUECOMPLETEARG)
         if sabnzbd.QUEUECOMPLETEARG:
@@ -571,9 +573,8 @@ def HandleEmptyQueue():
         else:
             Thread(target=sabnzbd.QUEUECOMPLETEACTION).start()
 
-        sabnzbd.QUEUECOMPLETEACTION = None
-        sabnzbd.QUEUECOMPLETEARG = None
         sabnzbd.QUEUECOMPLETEACTION_GO = False
+        sabnzbd.change_queue_complete_action(cfg.queue_complete.get(), new=False)
 
 
 def CleanUpList(wdir, skip_nzb):
@@ -592,7 +593,7 @@ def CleanUpList(wdir, skip_nzb):
                     os.remove(path)
                 except:
                     logging.error(Ta('error-ppDelFailed@1'), path)
-                    logging.debug("Traceback: ", exc_info = True)
+                    logging.info("Traceback: ", exc_info = True)
 
 
 def prefix(path, pre):
@@ -660,4 +661,4 @@ def remove_samples(path):
                     os.remove(path)
                 except:
                     logging.error(Ta('error-ppDelFailed@1'), path)
-                    logging.debug("Traceback: ", exc_info = True)
+                    logging.info("Traceback: ", exc_info = True)
