@@ -88,6 +88,11 @@ def clear_feed(feed):
 
 ################################################################################
 
+def notdefault(item):
+    """ Return True if not None/"None"/"Default"/""
+    """
+    return bool(item) and item.lower() not in ('default', 'none')
+
 
 def ListUris():
     """ Return list of all RSS uris """
@@ -104,21 +109,7 @@ def ConvertFilter(text):
     if text[:3].lower() == 're:':
         txt = text[3:]
     else:
-        txt = text.replace('\\','\\\\')
-        txt = txt.replace('^','\^')
-        txt = txt.replace('$','\$')
-        txt = txt.replace('.','\.')
-        txt = txt.replace('[','\[')
-        txt = txt.replace(']','\]')
-        txt = txt.replace('(','\(')
-        txt = txt.replace(')','\)')
-        txt = txt.replace('+','\+')
-        txt = txt.replace('?','\?')
-        txt = txt.replace('|','\|')
-        txt = txt.replace('{','\{')
-        txt = txt.replace('}','\}')
-        txt = txt.replace('*','.*')
-
+        txt = misc.wildcard_to_re(text)
     try:
         return re.compile(txt, re.I)
     except:
@@ -140,15 +131,18 @@ class RSSQueue:
                         if type(data) == type([]):
                             # Convert previous list-based store to dictionary
                             new = {}
-                            new['status'] = data[0]
-                            new['title'] = data[1]
-                            new['url'] = data[2]
-                            new['cat'] = data[3]
-                            new['pp'] = data[4]
-                            new['script'] = data[5]
-                            new['time'] = data[6]
-                            new['prio'] = str(NORMAL_PRIORITY)
-                            self.jobs[feed][link] = new
+                            try:
+                                new['status'] = data[0]
+                                new['title'] = data[1]
+                                new['url'] = data[2]
+                                new['cat'] = data[3]
+                                new['pp'] = data[4]
+                                new['script'] = data[5]
+                                new['time'] = data[6]
+                                new['prio'] = str(NORMAL_PRIORITY)
+                                self.jobs[feed][link] = new
+                            except IndexError:
+                                del new
                         else:
                             self.jobs[feed][link] = feeds[feed][link]
         except IOError:
@@ -178,15 +172,14 @@ class RSSQueue:
         """ Run the query for one URI and apply filters """
         self.shutdown = False
 
-        def DupTitle(fd, title):
-            for f in self.jobs:
-                if f == fd:
-                    for lk in self.jobs[fd]:
-                        item = self.jobs[fd][lk]
-                        if item.get('status', ' ')[0] == 'D' and \
-                           item.get('title', '').lower() == title.lower():
-                            return True
-                    return False
+        def DupTitle(title):
+            title = title.lower()
+            for fd in self.jobs:
+                for lk in self.jobs[fd]:
+                    item = self.jobs[fd][lk]
+                    if item.get('status', ' ')[0] == 'D' and \
+                       item.get('title', '').lower() == title:
+                        return True
             return False
 
 
@@ -200,7 +193,7 @@ class RSSQueue:
             feeds = config.get_rss()[feed]
         except KeyError:
             logging.error(Ta('error-rssBadFeed@1'), feed)
-            logging.debug("Traceback: ", exc_info = True)
+            logging.info("Traceback: ", exc_info = True)
             return T('error-rssBadFeed@1') % feed
 
         uri = feeds.uri.get()
@@ -244,6 +237,10 @@ class RSSQueue:
         if 'nzbs.org/' in uri and not ('&dl=1' in uri):
             uri += '&dl=1'
 
+        # Compemsate for crappy newzbin clone
+        if 'newzbin.com' in uri and not cfg.NEWZBIN_HTTPS.get():
+            uri = uri.replace('https:', 'http:')
+
         # Read the RSS feed
         logging.debug("Running feedparser on %s", uri)
         d = feedparser.parse(uri.replace('feed://', 'http://'))
@@ -258,7 +255,7 @@ class RSSQueue:
             return T('warn-failRSS@2') % (uri, str(d['bozo_exception']))
         if not entries:
             logging.info('RSS Feed was empty: %s', uri)
-            return 'RSS Feed was empty'
+            return ''
 
         order = 0
         # Filter out valid new links
@@ -266,12 +263,14 @@ class RSSQueue:
             if self.shutdown: return
 
             try:
-                link = _get_link(uri, entry)
+                link, category = _get_link(uri, entry)
             except (AttributeError, IndexError):
                 link = None
+                category = ''
                 logging.error('Incompatible feed %s', uri)
-                logging.debug("Traceback: ", exc_info = True)
+                logging.info("Traceback: ", exc_info = True)
                 return 'Incompatible feed'
+            category = latin1(category)
 
             if link:
                 # Make sure there are no spaces in the URL
@@ -283,7 +282,7 @@ class RSSQueue:
 
                 newlinks.append(link)
 
-                if cfg.NO_DUPES.get() and DupTitle(feed, title):
+                if cfg.NO_DUPES.get() and DupTitle(title):
                     logging.info("Ignoring duplicate job %s", atitle)
                     continue
 
@@ -292,51 +291,59 @@ class RSSQueue:
                 myScript = ''
                 #myPriority = 0
 
-                if (link not in jobs) or (jobs[link]['status'] in ('G', 'B', 'G*', 'B*')):
+                if (link not in jobs) or (jobs[link].get('status', ' ') in ('G', 'B', 'G*', 'B*')):
                     # Match this title against all filters
                     logging.debug('Trying title %s', atitle)
                     result = False
                     for n in xrange(regcount):
-                        found = re.search(regexes[n], title)
-                        if reTypes[n]=='M' and not found:
-                            logging.debug("Filter rejected on rule %d", n)
-                            result = False
-                            break
-                        if found and reTypes[n]=='A':
-                            logging.debug("Filter matched on rule %d", n)
-                            result = True
-                            if reCats[n]:
-                                myCat = reCats[n]
-                            else:
-                                myCat = defCat
-                            if rePPs[n]:
-                                myPP = rePPs[n]
-                            elif not reCats[n]:
-                                myPP = defPP
-                            if reScripts[n]:
-                                myScript = reScripts[n]
-                            elif not reCats[n]:
-                                myScript = defScript
-                            #elif not rePriority[n]:
-                                #myScript = defScript
-                            break
-                        if found and reTypes[n]=='R':
-                            logging.debug("Filter rejected on rule %d", n)
-                            result = False
-                            break
+                        if notdefault(reCats[n]):
+                            myCat = reCats[n]
+                        elif category:
+                            myCat = misc.CatConvert(category)
+                        else:
+                            myCat = defCat
+                        if notdefault(rePPs[n]):
+                            myPP = rePPs[n]
+                        elif not (reCats[n] or category):
+                            myPP = defPP
+                        if notdefault(reScripts[n]):
+                            myScript = reScripts[n]
+                        elif not (notdefault(reCats[n]) or category):
+                            myScript = defScript
+
+                        if category and reTypes[n]=='C':
+                            found = re.search(regexes[n], category)
+                            if not found:
+                                logging.debug("Filter rejected on rule %d", n)
+                                result = False
+                                break
+                        else:
+                            found = re.search(regexes[n], title)
+                            if reTypes[n]=='M' and not found:
+                                logging.debug("Filter rejected on rule %d", n)
+                                result = False
+                                break
+                            if found and reTypes[n]=='A':
+                                logging.debug("Filter matched on rule %d", n)
+                                result = True
+                                break
+                            if found and reTypes[n]=='R':
+                                logging.debug("Filter rejected on rule %d", n)
+                                result = False
+                                break
 
                     act = download and not first
                     if link in jobs:
-                        act = act and not jobs[link]['status'].endswith('*')
+                        act = act and not jobs[link].get('status', '').endswith('*')
                         act = act or force
-                        star = first or jobs[link]['status'].endswith('*')
+                        star = first or jobs[link].get('status', '').endswith('*')
                     else:
                         star = first
                     if result:
                         _HandleLink(jobs, link, title, 'G', myCat, myPP, myScript,
                                     act, star, order, priority=defPriority)
                     else:
-                        _HandleLink(jobs, link, title, 'B', defCat, defPP, defScript,
+                        _HandleLink(jobs, link, title, 'B', myCat, myPP, myScript,
                                     False, star, order, priority=defPriority)
             order += 1
 
@@ -348,7 +355,7 @@ class RSSQueue:
         olds  = jobs.keys()
         for old in olds:
             if old not in newlinks:
-                if jobs[old]['status'][0] in ('G', 'B'):
+                if jobs[old].get('status', ' ')[0] in ('G', 'B'):
                     jobs[old]['status'] = 'X'
                 try:
                     tm = float(jobs[old]['time'])
@@ -413,7 +420,7 @@ class RSSQueue:
             lst = self.jobs[feed]
             for link in lst:
                 if lst[link].get('url', '') == id:
-                    lst[link]['time'] = 'D'
+                    lst[link]['status'] = 'D'
 
     @synchronized(LOCK)
     def clear_feed(self, feed):
@@ -431,7 +438,10 @@ def _HandleLink(jobs, link, title, flag, cat, pp, script, download, star, order,
 
     jobs[link] = {}
     jobs[link]['order'] = order
-    nzbname = misc.sanitize_foldername(title)
+    if 'nzbindex.nl/' in link or 'nzbindex.com/' in link or 'nzbclub.com/' in link:
+        nzbname = None
+    else:
+        nzbname = misc.sanitize_foldername(title)
     m = RE_NEWZBIN.search(link)
     if m and m.group(1).lower() == 'newz' and m.group(2) and m.group(3):
         if download:
@@ -475,14 +485,19 @@ def _get_link(uri, entry):
     """ Retrieve the post link from this entry """
 
     link = None
+    category = ''
     uri = uri.lower()
-    if 'newzbin' in uri or 'newzxxx'in uri:
+    if 'newzbin.com' in uri or 'newzxxx.com'in uri:
         link = entry.link
         if not (link and '/post/' in link.lower()):
             # Use alternative link
             link = entry.links[0].href
     elif 'nzbindex.nl' in uri or 'nzbindex.com' in uri or 'animeusenet.org' in uri:
         link = entry.enclosures[0]['href']
+    elif 'feed.nzbclub.com' in uri:
+        link = entry.link
+        if link:
+            link = link.replace('nzb_view.aspx', 'nzb_download.aspx')
     elif not link:
         # Try standard link first
         link = entry.link
@@ -490,7 +505,14 @@ def _get_link(uri, entry):
             link = entry.links[0].href
 
     if link and 'http' in link.lower():
-        return link
+        try:
+            category = entry.cattext
+        except:
+            try:
+                category = entry.category
+            except:
+                category = ''
+        return link, category
     else:
         logging.warning(Ta('warn-emptyRSS@1'), link)
-        return None
+        return None, ''
